/* -------------------------------- PATH.C ---------------------------------
      Copyright (c) Spark Software Inc. 1990-1992.  All Rights Reserved.
   ------------------------------------------------------------------------- */

#include <windows.h>
#include <ddeml.h>
#include <string.h>
#include <memory.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "../commdefs.h"
#include "../commglob.h"

#define META_ASTERISK                   '*'
#define META_QUESTION                   '?'

PRIVATE BOOL  PASCAL IsMetacharacter( BYTE cChar );
PRIVATE LPSTR PASCAL GetFirstMetacharacter( LPSTR lpszSearchPath );

PRIVATE BYTE szMetacharacters[ ] = { META_ASTERISK , META_QUESTION , 0 };


PUBLIC BOOL PASCAL IsValidPath( LPSTR lpszPath ,
    LPSTR FAR *lplpszFirstMetaCharacter , LPSTR FAR *lplpszPathSeparator ,
    VOID FAR *lpStatInfo )
{
    PRIVATE BYTE szString[ 0x0200 ];
    int          nLength;
    int          nStatResult;
    struct _stat StatInfo;


    /* Does the path include metacharacters? */
    *lplpszFirstMetaCharacter = GetFirstMetacharacter( lpszPath );
    if( ( *lplpszPathSeparator = _fstrrchr( lpszPath , '/' ) ) ||
        ( *lplpszPathSeparator = _fstrrchr( lpszPath , '\\' ) ) ||
        ( *lplpszPathSeparator = _fstrchr( lpszPath , ':' ) ) )
    {
        if( *lplpszFirstMetaCharacter &&
            *lplpszFirstMetaCharacter < *lplpszPathSeparator )
        {
            /* Metacharacters only allowed in the */
            /* filename portion of the path.      */
            return FALSE;
        }
    }

    if( *lplpszFirstMetaCharacter )
    {
        if( *lplpszPathSeparator && *lplpszPathSeparator[ 0 ] != ':' )
        {
            /* Path with metacharacters has slashes. */
            /* Copy the path up to but not including */
            /* the last slash.                       */
            nLength = OFFSETOF( *lplpszPathSeparator ) -
                OFFSETOF( lpszPath );
            _fstrncpy( (LPSTR)szString , lpszPath , nLength );
            szString[ nLength ] = '\0';

            /* Root directory doesn't go to _stat( ).   */
            /* NOTE: we've already taken care of a root */
            /* directory without a drive designator.    */
            if( nLength == 2 && szString[ 1 ] == ':' )
            {
                szString[ 0 ] = '\0';
            }
        }
        else
        {
            /* Path with metacharacters has no slashes. */
            /* Cannot use this path for _stat( ).       */
            szString[ 0 ] = '\0';
        }
    }
    else
    {
        /* Path has no metacharacters. */
        /* Use the whole path.         */
        _fstrcpy( (LPSTR)szString , lpszPath );
    }

    /* Is the path a directory or a file (full pathname)? */
    nStatResult = _stat( szString , &StatInfo );
    *(struct _stat FAR *)lpStatInfo = StatInfo;
    if( szString[ 0 ] && nStatResult )
    {
        /* Couldn't get pathname info. */
        /* NOTE: we use this to catch bad paths    */
        /* with as well as without metacharacters. */
        return FALSE;
    }

    return TRUE;

}/* IsValidPath( ) */


PRIVATE BOOL PASCAL IsMetacharacter( BYTE cChar )
{
    register WORD wIndex;


    for( wIndex = 0; szMetacharacters[ wIndex ]; wIndex++ )
    {
        if( cChar == szMetacharacters[ wIndex ] )
        {
            return TRUE;
        }
    }
    return FALSE;

}/* IsMetacharacter( ) */


PRIVATE LPSTR PASCAL GetFirstMetacharacter( LPSTR lpszPath )
{
    while( *lpszPath )
    {
        if( IsMetacharacter( *lpszPath ) )
        {
            return lpszPath;
        }
        lpszPath++;
    }

    return (LPSTR)NULL;

}/* GetFirstMetacharacter( ) */

/* EOF */
